#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <math.h>
#include "lis.h"
#include "lis_matrix.h"

int lis_matrix_split(LIS_MATRIX A, LIS_VECTOR *D, LIS_MATRIX *L, LIS_MATRIX *U)
{
	int			i,j,n;
	int			err;

	switch( A->matrix_type )
	{
	case LIS_MATRIX_CRS:
		err = lis_matrix_split_crs(A,D,L,U);
		break;
	case LIS_MATRIX_CCS:
		break;
	case LIS_MATRIX_MSR:
		break;
	case LIS_MATRIX_DIA:
		break;
	case LIS_MATRIX_ELL:
		break;
	case LIS_MATRIX_JDS:
		break;
	case LIS_MATRIX_BSR:
		break;
	case LIS_MATRIX_BSC:
		break;
	case LIS_MATRIX_VBR:
		break;
	case LIS_MATRIX_COO:
		break;
	case LIS_MATRIX_DNS:
		break;
	}
	return err;
}

int lis_matrix_split_crs(LIS_MATRIX A, LIS_VECTOR *D, LIS_MATRIX *L, LIS_MATRIX *U)
{
	int				i,j,n;
	int				nnzl,nnzu;
	int				err;
	int				*lptr,*lindex,*uptr,*uindex;
	LIS_SCALAR		*lvalue,*uvalue;

	n      = A->n;
	nnzl   = 0;
	nnzu   = 0;
	lptr   = NULL;
	lindex = NULL;
	lvalue = NULL;
	uptr   = NULL;
	uindex = NULL;
	uvalue = NULL;

	for(i=0;i<n;i++)
	{
		for(j=A->ptr[i];j<A->ptr[i+1];j++)
		{
			if( A->index[j]<i )
			{
				nnzl++;
			}
			else if( A->index[j]>i )
			{
				nnzu++;
			}
		}
	}
	lptr = (int *)lis_malloc( (n+1)*sizeof(int) );
	if( lptr==NULL )
	{
		return LIS_OUT_OF_MEMORY;
	}
	uptr = (int *)lis_malloc( (n+1)*sizeof(int) );
	if( lptr==NULL )
	{
		lis_free2(6,lptr,lindex,lvalue,uptr,uindex,uvalue);
		return LIS_OUT_OF_MEMORY;
	}
	lindex = (int *)lis_malloc( nnzl*sizeof(int) );
	if( lindex==NULL )
	{
		lis_free2(6,lptr,lindex,lvalue,uptr,uindex,uvalue);
		return LIS_OUT_OF_MEMORY;
	}
	uindex = (int *)lis_malloc( nnzu*sizeof(int) );
	if( uindex==NULL )
	{
		lis_free2(6,lptr,lindex,lvalue,uptr,uindex,uvalue);
		return LIS_OUT_OF_MEMORY;
	}
	lvalue = (LIS_SCALAR *)lis_malloc( nnzl*sizeof(LIS_SCALAR) );
	if( lvalue==NULL )
	{
		lis_free2(6,lptr,lindex,lvalue,uptr,uindex,uvalue);
		return LIS_OUT_OF_MEMORY;
	}
	uvalue = (LIS_SCALAR *)lis_malloc( nnzu*sizeof(LIS_SCALAR) );
	if( uvalue==NULL )
	{
		lis_free2(6,lptr,lindex,lvalue,uptr,uindex,uvalue);
		return LIS_OUT_OF_MEMORY;
	}
	err = lis_vector_duplicateM(A,D);
	if( err )
	{
		lis_free2(6,lptr,lindex,lvalue,uptr,uindex,uvalue);
		return LIS_OUT_OF_MEMORY;
	}
	err = lis_matrix_create(n,0,A->comm,L);
	if( err )
	{
		lis_vector_destroy(*D);
		lis_free2(6,lptr,lindex,lvalue,uptr,uindex,uvalue);
		return LIS_OUT_OF_MEMORY;
	}
	err = lis_matrix_create(n,0,A->comm,U);
	if( err )
	{
		lis_vector_destroy(*D);
		lis_matrix_destroy(*L);
		lis_free2(6,lptr,lindex,lvalue,uptr,uindex,uvalue);
		return LIS_OUT_OF_MEMORY;
	}

	nnzl = 0;
	nnzu = 0;
	lptr[0] = 0;
	uptr[0] = 0;
	for(i=0;i<n;i++)
	{
		for(j=A->ptr[i];j<A->ptr[i+1];j++)
		{
#ifdef _MPI
			if( A->index[j]<i )
			{
				lindex[nnzl] = A->g_index[j];
				lvalue[nnzl] = A->value[j];
				nnzl++;
			}
#else
			if( A->index[j]<i )
			{
				lindex[nnzl] = A->index[j];
				lvalue[nnzl] = A->value[j];
				nnzl++;
			}
#endif
#ifdef _MPI
			else if( A->index[j]>i )
			{
				uindex[nnzu] = A->g_index[j];
				uvalue[nnzu] = A->value[j];
				nnzu++;
			}
#else
			else if( A->index[j]>i )
			{
				uindex[nnzu] = A->index[j];
				uvalue[nnzu] = A->value[j];
				nnzu++;
			}
#endif
			else
			{
				(*D)->value[i] = A->value[j];
			}
		}
		lptr[i+1] = nnzl;
		uptr[i+1] = nnzu;
	}
	lis_matrix_set_crs(nnzl,lptr,lindex,lvalue,*L);
	lis_matrix_assemble(L,LIS_MATRIX_CRS);
	lis_matrix_set_crs(nnzu,uptr,uindex,uvalue,*U);
	lis_matrix_assemble(U,LIS_MATRIX_CRS);
	return LIS_SUCCESS;
}
