#include "lis.h"
#include "lis_precon.h"

int lis_precon_create_jacobi(LIS_PRECON precon)
{
	int	i,j,n;
	int *diag;
	LIS_SCALAR *D;

	n    = precon->Amat->n;
	diag = precon->diag;
	D    = precon->D->value;
#pragma omp parallel for private(i,j)
	for(i=0; i<n; i++)
	{
		for(j=precon->Amat->ptr[i];j<precon->Amat->ptr[i+1];j++)
		{
			if( i==precon->Amat->index[j] )
			{
				diag[i] = j;
				D[i] = 1.0 / precon->Amat->value[j];
				break;
			}
		}
	}
	return LIS_SUCCESS;
}

void lis_psolve_jacobi(LIS_PRECON precon, LIS_VECTOR B, LIS_VECTOR X)
{
	int i,n;
	LIS_SCALAR *b,*x,*d;

	/*
	 *  Mx = b
	 *  M  = D
	 */

	n = precon->Amat->n;
	b = B->value;
	x = X->value;
	d = precon->D->value;

#pragma omp parallel for private(i)
	for(i=0; i<n; i++)
	{
		x[i] = b[i] * d[i];
	}
}

void lis_psolvet_jacobi(LIS_PRECON precon, LIS_VECTOR B, LIS_VECTOR X)
{
	int i,n;
	LIS_SCALAR *b,*x,*d;

	/*
	 *  M'x = b
	 *  M'  = D
	 */

	n = precon->Amat->n;
	b = B->value;
	x = X->value;
	d = precon->D->value;
#pragma omp parallel for private(i)
	for(i=0; i<n; i++)
	{
		x[i] = b[i] * d[i];
	}
}
